// Funes. Copyright (c) 2001, Nelson Minar <nelson@monkey.org>
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package org.monkey.nelson.funes;
import org.apache.log4j.Category;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

import dog.mail.mbox.MboxMessage;

import javax.mail.*;
import javax.mail.internet.*;
import java.util.Date;
import java.io.IOException;
import java.io.File;

/** Encapsulate the result of parsing a single Javamail message.
 ** @author Nelson Minar &lt;nelson@monkey.org&gt;
 **/
public class ParsedMessage extends Object {
    private static final Category L = Category.getInstance(ParsedMessage.class.getName());

    /** Parsing worked, getDocument() will return a Document object. **/
    public static final char STATUS_OK = '.';

    /** Parsing failed cleanly, getDocument() will return null. **/
    public static final char STATUS_PARSE_FAILURE = '*';

    /** Parsing threw an exception, getDocument() will return null. **/
    public static final char STATUS_EXCEPTION = 'E';

    /** This object is in an unknown status. **/
    public static final char STATUS_UNKNOWN = '?';

    protected char status = '?';
    protected Document document;
    protected Exception parseException;

    /** Create a new ParsedMessage object, by reading in a Javamail
     ** message.
     **/
    public ParsedMessage(Message m) {
        try {
            document = parseMessage(m);
        } catch (Exception ex) {
            this.status = STATUS_EXCEPTION;
            this.parseException = ex;
        }
    }

    /** Return a status code for how well the message was parsed.
     ** The status code is a character suitable for printing out.
     ** Four statuses are possible: STATUS_OK, STATUS_PARSE_FAILURE,
     ** STATUS_EXCEPTION, or STATUS_UNKNOWN
     **/
    public char getStatus() {
        return status;
    }

    /** Return the Lucene Document created by parsing the Message.
     ** This will return null if the message was not parsed correctly.
     **/
    public Document getDocument() {
        return document;
    }

    /** Return any Exception generated while parsing a message. This
     ** will return null unless the status is STATUS_EXCEPTION.
     **/
    public Exception getException() {
        return parseException;
    }

    /** Do the underlying parser work. **/
    protected Document parseMessage(Message m) throws Exception {
        Document doc = new Document();

        addField(doc, parseSubject(m));
        addField(doc, parseMessageID(m));
        addField(doc, parseAuthor(m));
        addField(doc, parseRecipients(m));
        addField(doc, parseDate(m));
        addField(doc, getLocation(m));
        
        Field bodyField = parseBody(m);
        if (bodyField == null) {
            status = STATUS_PARSE_FAILURE;
        } else {
            status = STATUS_OK;
        }
        
        addField(doc, bodyField);

        return doc;
    }

    /** Add a field to the document, but only if it is not null. **/
    protected void addField(Document d, Field f) {
        if (d != null && f != null)
            d.add(f);
    }

    /** Parse the body of the Javamail message. Some body types
     ** are not supported.
     ** @returns null if no useful body can be extracted.
     **/
    protected Field parseBody(Message m) throws IOException, MessagingException {
        Object content = m.getContent();
        if (!(content instanceof String)) {
            L.info("Body type " + content.getClass().getName() + " not indexed.");
            return null;
        }

        String body = (String) content;
        if (L.isDebugEnabled())
            L.debug("Body size: " + body.length());

        return Schema.makeBodyField(body);
    }

    /** Parse out the message subject. **/
    protected Field parseSubject(Message m) throws MessagingException {
        String subject = m.getSubject();
        if (subject == null) {
            L.info("Null subject");
            return null;
        } else {
            if (L.isDebugEnabled())
                L.debug("Subject: " + subject);
            return Schema.makeSubjectField(subject);
        }
    }

    /** Parse out the message ID. **/
    protected Field parseMessageID(Message m) throws MessagingException {
        String [] ids = m.getHeader("Message-ID");
        if (ids.length != 1)
            L.info("Expected 1 message ID, got " + ids.length);

        if (ids.length > 0) {
            if (L.isDebugEnabled())
                L.debug("Message ID: " + ids[0]);
            return Schema.makeMessageIDField(ids[0]);
        } else {
            return null;
        }
    }

    /** Parse out the author of the message and put it in canonical form. **/
    protected Field parseAuthor(Message m) throws MessagingException {
        Address [] authors = m.getFrom();
        if (authors.length != 1)
            L.info("Expected 1 author, got " + authors.length);

        if (authors.length > 0) {
            if (L.isDebugEnabled())
                L.debug("Author: " + authors[0]);
            return Schema.makeAuthorField(addressToString(authors[0]));
        } else {
            return null;
        }
    }

    /** Parse out all recipients of the message (To, Cc, etc) and
     ** put them in a comma separated list.
     **   Real Name <email@address>, Real Name2 <email@address2>, ...
     **/
    protected Field parseRecipients(Message m) throws MessagingException {
        StringBuffer recipients = new StringBuffer("");
        Address[] a = m.getAllRecipients();

        if (a == null || a.length < 1) {
            L.info("Found no recipients.");
            return null;
        }

        boolean needDelimiter = false;
        
        for (int i = 0; a != null && i < a.length; i++) {
            if (needDelimiter)
                recipients.append(", ");
            recipients.append(addressToString(a[i]));
            needDelimiter = true;
        }

        if (L.isDebugEnabled())
            L.debug("Adding recipients " + recipients.toString());
        return Schema.makeRecipientsField(recipients.toString());
    }

    /** Parse out the date the message was sent. **/
    protected Field parseDate(Message m) throws MessagingException {
        // Date d = m.getReceivedDate();           // This is always null?!
    
        Date d = m.getSentDate();
        if (d == null) {
            L.info("No sent date found.");
            return null;
        }

        if (L.isDebugEnabled())
            L.debug("Adding date " + d);

        return Schema.makeDateField(d);        
    }

    /** Get the location of this email message and store it as a
     ** x-filebyte: URL.
     ** @see org.nelson.monkey.protocol.x-filebyte
     **/
    protected Field getLocation(Message m) throws IOException {
        if (!(m instanceof MboxMessage)) {
            L.info("Cannot get location for message class " + m.getClass());
            return null;
        }

        MboxMessage mm = (MboxMessage) m;
        File fileObject = mm.getFileObject();
        long startOffset = mm.getStartOffset();
        long endOffset = mm.getEndOffset();
        String fileName = fileObject.getCanonicalPath();

        String url = "xfilerange://localhost" + fileName + "#" + startOffset + "-" + endOffset;
        if (L.isDebugEnabled())
            L.debug("Adding location " + url);
        return Schema.makeLocationField(url);
    }
    
    /** Convert a JavaMail address to a canonical string
     ** representation. Done by simply lining up the email address and
     ** personal name together.
     **/
    protected String addressToString(Address a) {
        if (!(a instanceof InternetAddress)) {
            L.info("Not InternetAddress: " + a);
            return null;
        }
        InternetAddress ia = (InternetAddress)a;
        StringBuffer sb = new StringBuffer();

        boolean needSpace = false;

        // Get the email address out
        if (ia.getAddress() != null) {
            sb.append(ia.getAddress());
            needSpace = true;
        }

        // Get the person's name. Also stick in a space if there was an
        // email address.
        if (ia.getPersonal() != null) {
            if (needSpace)
                sb.append(' ');
            sb.append('(');
            sb.append(ia.getPersonal());
            sb.append(')');
        }

        return sb.toString();
    }
}
