// Funes. Copyright (c) 2001, Nelson Minar <nelson@monkey.org>
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package org.monkey.nelson.funes;
import org.apache.log4j.Category;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.DateField;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;

import java.util.Date;

/** The Schema class encapsulates details of the schema used to store
 ** data in Lucene. Summary:
 **   subject
 **   author
 **   recipients
 **   date        untokenized (stored in Lucene format)
 **   message ID  untokenized
 **   body        unstored (must be retrieved from message)
 **   location    unindexed (not searchable)
 **
 ** @author Nelson Minar &lt;nelson@monkey.org&gt;
 **/
public class Schema extends Object {
    private static final Category L = Category.getInstance(Schema.class.getName());

    /** Make a new analyzer to use for indexing or querying. **/
    public static Analyzer makeAnalyzer() {
        return new StandardAnalyzer();
    }
    
    /** Field name for the message body contents.
     **  tokenized, indexed   Field.UnStored()
     **/
    public static final String bodyName = "body";

    /** Field name for the email subject.
     ** tokenized, indexed, stored.  Field.Text()
     **/
    public static final String subjectName = "subject";

    /** Field name for the email message ID.
     ** indexed, stored.  Field.Keyword()
     **/
    public static final String messageIDName = "messageID";

    /** Field name for author of email. Email address plus personal name.
     ** tokenized, indexed, stored.  Field.Text()
     **/
    public static final String authorName = "author";

    /** Field name for recipients of email. Addresses plus personal names.
     ** tokenized, indexed, stored.  Field.Text()
     **/
    public static final String recipientsName = "recipients";

    /** Field name for date email was received. String encoded as DateField
     ** indexed, stored. Field.Keyword() (encoded by DateField).
     **/
    public static final String dateName = "date";

    /** Field name for where this email is located. A URI.
     ** stored.  Field.UnIndexed()
     **/
    public static final String locationName = "location";

    public static Field makeMessageIDField(String messageID) {
        return Field.Keyword(Schema.messageIDName, messageID);
    }

    public static Field makeBodyField(String body) {
        return Field.UnStored(Schema.bodyName, body);
    }

    public static Field makeSubjectField(String subject) {
        return Field.Text(Schema.subjectName, subject);
    }

    public static Field makeAuthorField(String author) {
        return Field.Text(Schema.authorName, author);
    }

    public static Field makeRecipientsField(String recipients) {
        return Field.Text("recipients", recipients);
    }

    /** Make a field to store a Java date. This is stored in Lucene's
     ** own funky date format, a string encoding which sorts well.
     ** @see org.apache.lucene.document.DateField
     **/
    public static Field makeDateField(Date d) {
        String ds = DateField.dateToString(d);
        return Field.Keyword(Schema.dateName, ds);
    }

    public static Field makeLocationField(String url) {
        return Field.UnIndexed(Schema.locationName, url);
    }

    /** Return a Java Date from the format used in the index. **/
    public static Date parseDateField(String d) {
        return DateField.stringToDate(d);
    }
}
