// Funes. Copyright (c) 2001, Nelson Minar <nelson@monkey.org>
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

// Copyright (c) 2001, Nelson Minar <nelson@monkey.org>. All rights reserved.

package org.monkey.nelson.protocol.xfilerange;
import org.apache.log4j.Category;

import java.io.InputStream;
import java.io.IOException;
import java.io.EOFException;

/** A wrapper for other input streams that limits the reading to a given
 ** byte range. If start is specified, that many bytes are skipped before
 ** the first read. If end is specified, EOF is returned when that byte is
 ** read; if EOF occurs before end, EOFException is thrown.
 **
 ** Optimizations to make:
 **   implement efficient block reads
 **   implement skip()
 **
 ** This class is deliberately <b>not</b> a subclass of
 ** java.io.FilterInputStream; that class seems to break the relationship
 ** between read(byte[]) and read().
 **
 ** @author Nelson Minar &lt;nelson@monkey.org&gt;
 **/
public class ByteRangeInputStream extends InputStream {
    private static final Category L = Category.getInstance(ByteRangeInputStream.class.getName());

    /** Was an end specified? **/
    protected boolean endSet = false;

    /** If so, which byte is the last we should read? **/
    protected long end;

    /** The underlying input stream. **/
    protected InputStream is;

    /** How many bytes have been read from the stream so far, including
     ** those we skipped at the start.
     **/
    protected long bytesRead = 0;

    /** Construct an input stream that will read out the underlying input
     ** stream, having skipped <i>start</i> bytes.
     **/
    public ByteRangeInputStream(InputStream is, long start) throws IOException {
        this.is = is;
        skip(start);
    }

    /** Construct an input stream that will read out the underlying input
     ** stream, having skipped <i>start</i> bytes, until <i>end</i> bytes.
     ** <i>end</i> is not a number of bytes to read, it is the ending index.
     ** This class guarantees that <i>end</i>-<i>start</i> bytes will be
     ** returned, or else an EOFException will be generated.
     **/
    public ByteRangeInputStream(InputStream is, long start, long end) throws IOException, IllegalArgumentException {
        this(is, start);

        if (end < start)
            throw new IllegalArgumentException("End is before start!");

        this.end = end;
        this.endSet = true;
    }

    /** Read a single byte off the stream.
     ** If an end was specified, then bytes only up to <i>end</i> will
     ** be returned, and then EOF will be returned. In addition, if
     ** an end was specified and EOF has been reached on the underlying
     ** stream before the appropriate number of bytes are read, an
     ** EOFException is thrown.
     **/
    public synchronized int read() throws IOException {
        if (!endSet) {
            // Don't care about ending, just pass through underlying stream.
            int c = is.read();
            bytesRead++;
            return c;
        } else {
            // Have we already read too many bytes? Return EOF.
            if (bytesRead >= end)
                return -1;

            // OK, read a byte, count it.
            int c = is.read();
            bytesRead++;

            // Did we get a premature EOF? Throw exception.
            if (c == -1 && bytesRead < end)
                throw new EOFException("Ran out of bytes to read before designated end of range.");

            return c;
        }
    }
}
