// Funes. Copyright (c) 2001, Nelson Minar <nelson@monkey.org>
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package org.monkey.nelson.protocol.xfilerange;
import org.apache.log4j.Category;

import java.util.Properties;
import java.net.URLStreamHandler;
import java.net.URLConnection;
import java.net.URL;
import java.io.IOException;

/** Class that handles "xfilerange:" style URLs. This is a simple
 ** augmentation of the standard file:// URL which allows a byte range
 ** in the file to be named. Ie:
 **   xfilerange://etc/fstab#32-131
 ** refers to the file /etc/fstab, but only bytes 32-131.
 **
 ** Details of the syntax:
 **   Everything up to the # is exactly like the file: URL.
 **   After the # (the "reference", or "anchor name") is a byte range.
 **     #start-end
 **     Byte 0 is the first byte in the file.
 **     If start is omitted, byte 0 is assumed
 **     If end is omitted, read-until-EOF is assumed
 **     If end is beyond EOF, an IO exception will occur when those bytes
 **       are read. (Ie: read-until-EOF is *not* assumed)
 **     If both start and end are omitted, or if there is no reference,
 **       then xfilerange: is identical to file:
 **
 ** To install this handler, call the install() method.
 **
 ** @see http://www.w3.org/Addressing/rfc1738.txt
 ** @see http://www.ietf.org/rfc/rfc2717.txt
 ** @see http://www.ietf.org/rfc/rfc2718.txt
 ** @author Nelson Minar &lt;nelson@monkey.org&gt;
 **/
public class Handler extends URLStreamHandler {
    private static final Category L = Category.getInstance(Handler.class.getName());
    private static boolean installed = false;

    /** Package above the URL handler package. **/
    private static final String rootPackageName = "org.monkey.nelson.protocol";

    /** Install the handler for xfilerange: URLs so that Java will
     ** be able to resolve them.
     **/
    public synchronized static void install() {
        if (installed)
            return;

        L.debug("Installing xfilerange: handler");
        // Read the magic Java system property that stores handlers.
        String propertyName = "java.protocol.handler.pkgs";
        Properties p = System.getProperties();

        // Add ourself in - appending if necessary.
        String oldValue = p.getProperty(propertyName);
        String newValue;
        if (oldValue == null)
            newValue = rootPackageName;
        else
            newValue = oldValue + "|" + rootPackageName;

        // Put the handler value back. It's not clear this write is
        // strictly allowed by Java, but it seems to work.
        p.put(propertyName, newValue);
    }

    // No special implementation of parse URL - XFileRangeConnection
    // will do most of that work.
    
    /** Open a connection to the given xfilerange URL. **/
    protected URLConnection openConnection(URL u) throws IOException {
        return new XFileRangeURLConnection(u);
    }
}
