// Funes. Copyright (c) 2001, Nelson Minar <nelson@monkey.org>
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package org.monkey.nelson.protocol.xfilerange;
import org.apache.log4j.Category;
import java.io.*;
import java.net.URLConnection;
import java.net.URL;
import java.net.MalformedURLException;

/** Class that implements connections to xfilerange: URLs.
 **
 ** @see Handler for details on URL syntax.
 ** @author Nelson Minar &lt;nelson@monkey.org&gt;
 **/
public class XFileRangeURLConnection extends URLConnection {
    private static final Category L = Category.getInstance(XFileRangeURLConnection.class.getName());

    /** The input stream for our underlying file. **/
    protected InputStream ourInputStream;

    /** Whether we've already connected to the underlying file. **/
    protected boolean connected;

    /** Our URL. **/
    protected URL url;

    protected XFileRangeURLConnection(URL u) {
        super(u);
        this.url = u;
        L.debug("Constructing URL connection for " + u);
    }

    /** Connect to an xfilerange. Opens the underlying file, then
     ** constructs a special input stream to narrow the range.
     **/
    public void connect() throws IOException {
        String fileURLName = "file://" + url.getHost() + "/" + url.getFile();
        String range = url.getRef();
        L.debug("Connecting to " + fileURLName + " for range " + range);

        URL fileURL = new URL(fileURLName);
        InputStream fileInputStream = fileURL.openStream();

        if (range == null || range.equals("") || range.equals("-")) {
            // No range requested? Just return the file input stream.
            ourInputStream = fileInputStream;
        } else {
            int dash = range.indexOf('-');
            String startString = range.substring(0, dash);
            String endString = range.substring(dash+1, range.length());

            boolean startSet = false;
            long start = -1;
            boolean endSet = false;
            long end = -2;
            
            try {
                if (!("".equals(startString))) {
                    start = Long.parseLong(startString);
                    startSet = true;
                }
                if (!("".equals(endString))) {
                    end = Long.parseLong(endString);
                    endSet = true;
                }
            } catch (NumberFormatException nfe) {
                throw new MalformedURLException("Reference " + range + " not a valid range.");
            }

            if (startSet && endSet) {
                ourInputStream = new ByteRangeInputStream(fileInputStream, start, end);
            } else if (startSet && !endSet) {
                ourInputStream = new ByteRangeInputStream(fileInputStream, start);
            } else if (!startSet && endSet) {
                ourInputStream = new ByteRangeInputStream(fileInputStream, 0, end);
            } else {
                ourInputStream = fileInputStream;
            }
        }
        
        connected = true;
    }

    /** Get the input stream for the xfilerange URL. **/
    public InputStream getInputStream() throws IOException {
        if (!connected)
            connect();
        return ourInputStream;
    }
}
