// Funes. Copyright (c) 2001, Nelson Minar <nelson@monkey.org>
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package org.monkey.nelson.protocol.xfilerange.test;
import junit.framework.*;
import org.monkey.nelson.protocol.xfilerange.*;
import java.io.*;
import java.net.*;

/**
 ** @author Nelson Minar &lt;nelson@monkey.org&gt;
 **/
public class ByteRangeInputStreamTest extends TestCase {
    public ByteRangeInputStreamTest(String name) { super(name); }
    public static Test suite() { return new TestSuite(ByteRangeInputStreamTest.class); }

    private String testString = "123456789";
    private InputStream baseIS;
    
    public void setUp() throws Exception {
        Handler.install();
        baseIS = new ByteArrayInputStream(testString.getBytes());
    }

    public void testFullRead() throws Exception {
        InputStream is = new ByteRangeInputStream(baseIS, 0);
        assertEquals(testString, readFully(is));
    }

    public void testMiddleToEnd() throws Exception {
        InputStream is = new ByteRangeInputStream(baseIS, 3);
        assertEquals(testString.substring(3), readFully(is));
    }

    public void testBeginToMiddle() throws Exception {
        InputStream is = new ByteRangeInputStream(baseIS, 0, 5);
        assertEquals(testString.substring(0, 5), readFully(is));
    }

    public void testMidToMid() throws Exception {
        InputStream is = new ByteRangeInputStream(baseIS, 3, 5);
        String r = readFully(is);
        assertEquals(r.length(), 2);
        assertEquals(testString.substring(3, 5), r);
    }

    public void testMidToMidOneAtATime() throws Exception {
        InputStream is = new ByteRangeInputStream(baseIS, 3, 5);
        String r = readFullyOneAtATime(is);
        assertEquals(r.length(), 2);
        assertEquals(testString.substring(3, 5), r);
    }

    public void testOverrun() throws Exception {
        InputStream is = new ByteRangeInputStream(baseIS, 3, testString.length() + 5);
        try {
            readFully(is);
            fail("Should have thrown EOFException");
        } catch (EOFException ex) {
        }
    }

    public void testStartOverrun() throws Exception {
        InputStream is = new ByteRangeInputStream(baseIS, testString.length() + 5, testString.length() + 15);
        try {
            readFully(is);
            fail("Should have thrown EOFException");
        } catch (EOFException ex) {
        }
    }

    public void testLengthZero() throws Exception {
        InputStream is = new ByteRangeInputStream(baseIS, 3, 3);
        assertEquals(0, readFully(is).length());
    }

    public void testBadArg() throws Exception {
        try {
            new ByteRangeInputStream(baseIS, 10, 3);
            fail("Should have thrown an IllegalArgumentException.");
        } catch (IllegalArgumentException ex) {
        }
    }

    public static String readFully(InputStream is) throws IOException {
        ByteArrayOutputStream os = new ByteArrayOutputStream(16384);
        byte [] buf = new byte[1024];
        int r;

        while ((r = is.read(buf)) >= 0) {
            os.write(buf, 0, r);
        }
        return os.toString();
    }

    // read a stream fully, one byte at a time. Doesn't catch bugs as
    // well as block read.
    public static String readFullyOneAtATime(InputStream is) throws IOException {
        StringBuffer b = new StringBuffer(1024);
        int c;
        while ((c = is.read()) != -1)
            b.append((char)c);
        return b.toString();
    }
}
